#!/usr/bin/env python

## @package ExamplePlugin
# Example Plugin Module.
# Every single plugin needs to come as a module in a directory.
# Every folder containing a file named "FOLDERNAME.info"
# will be recogniced as a plugin.
# Furthermore the folder needs to contain a file "__init__.py"
# which holds a class named "main".
# When loading a plugin pyjama will pass the object "pyjama"
# which holds all the other objects.
#
# For more informations have a look at the <a href = "Plugins.html"><b>plugin interface.</b></a> 

try:
    import gtk
except:
    print "Error: Gtk not found"
    raise

## Example Plugin class main called by Pyjama
class main():
    ## The Constructor
    # @param self Object pointer
    # @param pyjama Reference to the pyjama object
    def __init__(self, pyjama):
        ## Here we store the pyjama object reference
    	self.pyjama = pyjama

        ## You cannot directly access the plugins
        # object of pyjama in your init function
        # since it is created after all plugins
        # have been initialised.
        # Following line will fail:\n
        # <i>plugins = self.pyjama.plugins</i> <-- ERROR\n
        # Connect to the <a href = "Events.html">"alldone"</a> event, if you
        # want pyjama to call your plugin after
        # everythin else is done!
        self.Events = self.pyjama.Events
        self.Events.connect_event("alldone", self.ev_alldone)

        # You can diconnect fom events, but not
        # when the function, which disconnects from
        # the event was called by the event you want
        # to disconnect from.
        # This might be a bigger issue...
        #self.Events.disconnect_event("alldone", self.ev_alldone)       

    ## This function will be called after pyjama
    # has been loaded and you have connected to
    # the "alldone" event
    # Now you can access any other plugins...
    def ev_alldone(self):
        if "nowplaying" in self.pyjama.plugins.loaded:
            # print "Plugin 'nowplaying' can be found!'
            pass 

    ## The class "Preferences" is called with pyjama as param
    # when you set "haspreferences=True" in your plugin.info
    # This class is a subclass of our plugin
    class Preferences():
        ## Preferences Constructor
        # @param self Object Pointer
        # @param pyjama Again we have a pyjama object reference passed to our class
        def __init__(self, pyjama):
            me = pyjama.plugins.plugins_by_name["example"]
            dialog = gtk.MessageDialog(pyjama.window, 0, gtk.MESSAGE_INFO, gtk.BUTTONS_OK, "This is an exmaple preferences dialog\nor at least could be one.\nI can tell you something about me:\n\nAuthor: %s\nName: %s\nDescription: %s" % (me['author'], me['name'], me['description']))
            dialog.run()
            dialog.destroy()
